@@ -45,48 +45,30 @@ exm_application_new (gchar             *application_id,
                          NULL);
 }
 
-static void
-save_window_state (GtkWindow *self,
-                   gpointer   user_data G_GNUC_UNUSED)
-{
-    GSettings *settings;
-    gint default_width, default_height;
-    gboolean maximized;
-
-    settings = g_settings_new (APP_ID);
-
-    g_object_get (self, "default-width", &default_width, NULL);
-    g_object_get (self, "default-height", &default_height, NULL);
-    g_object_get (self, "maximized", &maximized, NULL);
-
-    g_settings_set_int (settings, "width", default_width);
-    g_settings_set_int (settings, "height", default_height);
-    g_settings_set_boolean (settings, "is-maximized", maximized);
-
-    g_object_unref (settings);
-}
-
 static ExmWindow *
 get_current_window (GApplication *app)
 {
-    GSettings *settings;
     GtkWindow *window;
-
-    settings = g_settings_new (APP_ID);
+    GSettings *settings;
 
     /* Get the current window or create one if necessary. */
     window = gtk_application_get_active_window (GTK_APPLICATION (app));
     if (window == NULL)
-    {
         window = g_object_new (EXM_TYPE_WINDOW,
                                "application", app,
-                               "default-width", g_settings_get_int (settings, "width"),
-                               "default-height", g_settings_get_int (settings, "height"),
-                               "maximized", g_settings_get_boolean (settings, "is-maximized"),
                                NULL);
 
-        g_signal_connect (window, "close-request", G_CALLBACK (save_window_state), NULL);
-    }
+    settings = g_settings_new (APP_ID);
+
+    g_settings_bind (settings, "width",
+                     window, "default-width",
+                     G_SETTINGS_BIND_DEFAULT);
+    g_settings_bind (settings, "height",
+                     window, "default-height",
+                     G_SETTINGS_BIND_DEFAULT);
+    g_settings_bind (settings, "is-maximized",
+                     window, "maximized",
+                     G_SETTINGS_BIND_DEFAULT);
 
     g_object_unref (settings);
 
@@ -140,7 +122,7 @@ exm_application_open (GApplication  *app,
     }
 
     scheme = g_uri_get_scheme (uri);
-    if (!g_str_equal (scheme, "gnome-extensions"))
+    if (g_strcmp0 (scheme, "gnome-extensions") != 0)
     {
         g_critical ("Invalid URI scheme: '%s'\n", scheme);
         return;
@@ -155,13 +137,6 @@ exm_application_open (GApplication  *app,
     gtk_widget_activate_action (GTK_WIDGET (window), "win.show-detail", "s", uuid);
 }
 
-static void
-exm_application_shutdown (GApplication *app)
-{
-    save_window_state (GTK_WINDOW (get_current_window (app)), NULL);
-
-    G_APPLICATION_CLASS (exm_application_parent_class)->shutdown (app);
-}
 
 static void
 exm_application_class_init (ExmApplicationClass *klass)
@@ -176,7 +151,6 @@ exm_application_class_init (ExmApplicationClass *klass)
     */
     app_class->activate = exm_application_activate;
     app_class->open = exm_application_open;
-    app_class->shutdown = exm_application_shutdown;
 }
 
 static void
