@@ -29,6 +29,7 @@
 #include "web/exm-search-provider.h"
 #include "web/model/exm-search-result.h"
 
+#include <adwaita.h>
 #include <glib/gi18n.h>
 
 struct _ExmBrowsePage
@@ -41,7 +42,6 @@ struct _ExmBrowsePage
 
     GtkStringList *suggestions;
     GListModel *search_results_model;
-    gchar *shell_version;
 
     int current_page;
     int max_pages;
@@ -49,6 +49,7 @@ struct _ExmBrowsePage
     GCancellable *cancellable;
 
     // Template Widgets
+    AdwBreakpoint       *breakpoint;
     AdwStatusPage       *status_page;
     GtkSearchEntry      *search_entry;
     GtkListBox          *search_results;
@@ -123,27 +124,18 @@ exm_browse_page_set_property (GObject      *object,
     }
 }
 
-GtkSearchEntry *
-exm_browse_page_get_search_entry (ExmBrowsePage *self)
-{
-  return self->search_entry;
-}
-
 static GtkWidget *
 search_widget_factory (ExmSearchResult *result,
                        ExmBrowsePage   *self)
 {
     ExmSearchRow *row;
-    gchar *uuid;
-    gboolean is_installed;
-    gboolean is_supported;
+    GValue value = G_VALUE_INIT;
 
-    g_object_get (result, "uuid", &uuid, NULL);
+    row = exm_search_row_new (self->manager, result);
 
-    is_installed = exm_manager_is_installed_uuid (self->manager, uuid);
-    is_supported = exm_search_result_supports_shell_version (result, self->shell_version);
-
-    row = exm_search_row_new (result, is_installed, is_supported);
+    g_value_init (&value, G_TYPE_BOOLEAN);
+    g_value_set_boolean (&value, TRUE);
+    adw_breakpoint_add_setter (self->breakpoint, G_OBJECT (row), "compact", &value);
 
     return GTK_WIDGET (row);
 }
@@ -191,12 +183,8 @@ on_first_page_result (GObject       *source,
 
     if (error)
     {
-        // Filter 5xx status codes (server errors)
-        if (error->code / 100 == 5)
-            gtk_label_set_markup (self->error_label, _("Check <a href='https://status.gnome.org/'>GNOME infrastructure status</a> and try again later"));
-        else
-            gtk_label_set_text (self->error_label, _("Check your network status and try again"));
-
+        gtk_label_set_text (self->error_label, error->message);
+        gtk_label_set_use_markup (self->error_label, TRUE);
         gtk_stack_set_visible_child_name (self->search_stack, "page_error");
 
         g_clear_error (&error);
@@ -308,9 +296,25 @@ search (ExmBrowsePage *self,
                                      self);
 }
 
-static void
-on_search_changed (ExmBrowsePage  *self)
+void
+exm_browse_page_search (ExmBrowsePage *self,
+                        const gchar   *query)
 {
+    gtk_editable_set_text (GTK_EDITABLE (self->search_entry), query);
+}
+
+void
+exm_browse_page_focus_entry (ExmBrowsePage *self)
+{
+    gtk_widget_grab_focus (GTK_WIDGET (self->search_entry));
+}
+
+static void
+on_search_changed (ExmBrowsePage *self)
+{
+    if (!gtk_widget_get_realized (GTK_WIDGET (self->search_entry)))
+        return;
+
     const char *query = gtk_editable_get_text (GTK_EDITABLE (self->search_entry));
     ExmSearchSort sort = (ExmSearchSort) gtk_drop_down_get_selected (self->search_dropdown);
     search (self, query, sort);
@@ -356,7 +360,6 @@ on_bind_manager (ExmBrowsePage *self)
                   &shell_version,
                   NULL);
 
-    self->shell_version = shell_version;
     g_object_set (self->search, "shell-version", shell_version, NULL);
 
     refresh_search (self);
@@ -417,6 +420,7 @@ exm_browse_page_class_init (ExmBrowsePageClass *klass)
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
     gtk_widget_class_set_template_from_resource (widget_class, g_strdup_printf ("%s/exm-browse-page.ui", RESOURCE_PATH));
+    gtk_widget_class_bind_template_child (widget_class, ExmBrowsePage, breakpoint);
     gtk_widget_class_bind_template_child (widget_class, ExmBrowsePage, status_page);
     gtk_widget_class_bind_template_child (widget_class, ExmBrowsePage, search_entry);
     gtk_widget_class_bind_template_child (widget_class, ExmBrowsePage, search_results);
@@ -448,14 +452,14 @@ exm_browse_page_init (ExmBrowsePage *self)
                      self->search, "show-unsupported",
                      G_SETTINGS_BIND_GET);
 
+    g_object_unref (settings);
+
     // Rerun search when show unsupported is toggled
     g_signal_connect_swapped (self->search,
                               "notify::show-unsupported",
                               G_CALLBACK (on_search_changed),
                               self);
 
-    g_object_unref (settings);
-
     load_suggestions (self);
 
     gtk_search_entry_set_key_capture_widget (self->search_entry, GTK_WIDGET (self));
