@@ -59,6 +59,7 @@ struct _ExmDetailView
     AdwHeaderBar *header_bar;
     AdwWindowTitle *title;
     GtkStack *stack;
+    GtkLabel *error_label;
     AdwStatusPage *error_status;
     GtkButton *ext_install;
     GtkLabel *ext_description;
@@ -66,12 +67,12 @@ struct _ExmDetailView
     GtkLabel *ext_title;
     GtkLabel *ext_author;
     ExmScreenshot *ext_screenshot;
-	GtkOverlay *ext_screenshot_container;
-	GtkButton *ext_screenshot_popout_button;
+    GtkOverlay *ext_screenshot_container;
+    GtkButton *ext_screenshot_popout_button;
     ExmInfoBar *ext_info_bar;
     GtkScrolledWindow *scroll_area;
     GtkStack *comment_stack;
-    GtkFlowBox *comment_box;
+    AdwWrapBox *comment_box;
     GtkButton *show_more_btn;
 
     AdwActionRow *link_homepage;
@@ -82,7 +83,8 @@ struct _ExmDetailView
     AdwActionRow *link_extensions;
     gchar *uri_extensions;
     int pk;
-    guint signal_id;
+    guint comments_signal_id;
+    guint install_signal_id;
 };
 
 G_DEFINE_FINAL_TYPE (ExmDetailView, exm_detail_view, ADW_TYPE_NAVIGATION_PAGE)
@@ -209,17 +211,13 @@ queue_resolve_image (ExmDetailView    *self,
                                       g_object_ref (self));
 }
 
-static GtkWidget *
-comment_factory (ExmComment *comment,
-                 gpointer    user_data G_GNUC_UNUSED)
+static void
+delete_comment_tiles (ExmDetailView *self)
 {
-    GtkWidget *tile;
+  GtkWidget *child;
 
-    tile = gtk_flow_box_child_new ();
-    gtk_widget_add_css_class (tile, "card");
-    gtk_flow_box_child_set_child (GTK_FLOW_BOX_CHILD (tile), GTK_WIDGET (exm_comment_tile_new (comment)));
-
-    return tile;
+  while ((child = gtk_widget_get_first_child (GTK_WIDGET (self->comment_box))) != NULL)
+    adw_wrap_box_remove (self->comment_box, child);
 }
 
 static void
@@ -233,19 +231,41 @@ on_get_comments (GObject       *source,
 
     if (error != NULL)
     {
-        gtk_stack_set_visible_child_name (self->comment_stack, "page_error");
-        g_critical ("An issue occurred while loading comments: %s", error->message);
+        if (error->domain == g_quark_try_string ("request-error-quark") && error->code == 403)
+        {
+            gtk_stack_set_visible_child_name (self->comment_stack, "page_disabled");
+        }
+        else
+        {
+            gtk_stack_set_visible_child_name (self->comment_stack, "page_error");
+            gtk_label_set_label (self->error_label, error->message);
+        }
+
         return;
     }
 
-    if (g_list_model_get_n_items (model) == 0)
-        gtk_stack_set_visible_child_name (self->comment_stack, "page_empty");
-    else
-        gtk_stack_set_visible_child_name (self->comment_stack, "page_comments");
+    guint n_comments = g_list_model_get_n_items (model);
 
-    gtk_flow_box_bind_model (self->comment_box, model,
-                             (GtkFlowBoxCreateWidgetFunc) comment_factory,
-                             g_object_ref (self), g_object_unref);
+    if (!n_comments)
+    {
+        gtk_stack_set_visible_child_name (self->comment_stack, "page_empty");
+    }
+    else
+    {
+        gtk_widget_set_visible (GTK_WIDGET (self->show_more_btn), n_comments >= 5);
+        gtk_stack_set_visible_child_name (self->comment_stack, "page_comments");
+    }
+
+    delete_comment_tiles (self);
+
+    for (guint i = 0; i < g_list_model_get_n_items (model); i++)
+    {
+        ExmComment *comment = g_list_model_get_item (model, i);
+        GtkWidget *tile = GTK_WIDGET (exm_comment_tile_new (comment));
+        gtk_widget_add_css_class (tile, "card");
+        adw_wrap_box_append (self->comment_box, tile);
+        g_object_unref (comment);
+    }
 }
 
 static void
@@ -260,7 +280,7 @@ queue_resolve_comments (ExmDetailView *self,
 }
 
 static void
-show_more_comments (GtkButton *button G_GNUC_UNUSED,
+show_more_comments (GtkButton     *button G_GNUC_UNUSED,
                     ExmDetailView *self)
 {
     GtkRoot *toplevel;
@@ -272,6 +292,14 @@ show_more_comments (GtkButton *button G_GNUC_UNUSED,
     adw_dialog_present (ADW_DIALOG (dlg), GTK_WIDGET (toplevel));
 }
 
+static void
+on_install_status (ExmManager            *manager G_GNUC_UNUSED,
+                   ExmInstallButtonState  state,
+                   ExmDetailView         *self)
+{
+    g_object_set (self->ext_install, "state", state, NULL);
+}
+
 static void
 install_remote (GtkButton     *button,
                 ExmDetailView *self)
@@ -282,25 +310,17 @@ install_remote (GtkButton     *button,
     g_object_get (self->ext_install, "state", &state, NULL);
 
     warn = (state == EXM_INSTALL_BUTTON_STATE_UNSUPPORTED);
+
+    g_object_set (self->ext_install, "state", EXM_INSTALL_BUTTON_STATE_INSTALLING, NULL);
+
+    // Move focus to previous widget to avoid losing it after the button becomes insensitive
+    gtk_widget_grab_focus (GTK_WIDGET (self->ext_author));
+
     gtk_widget_activate_action (GTK_WIDGET (button),
                                 "ext.install",
                                 "(sb)", self->uuid, warn);
 }
 
-static void
-delete_donation_rows (ExmDetailView *self)
-{
-    GtkWidget *row;
-
-    for (GList *iter = self->donation_rows_list; iter != NULL; iter = g_list_next (iter)) {
-        row = GTK_WIDGET (iter->data);
-        adw_expander_row_remove (self->links_donations, row);
-    }
-
-    g_list_free (self->donation_rows_list);
-    self->donation_rows_list = NULL;
-}
-
 static void
 new_donation_row (ExmDetailView *self,
                   int            num_donation)
@@ -317,7 +337,6 @@ new_donation_row (ExmDetailView *self,
     gtk_actionable_set_action_target_value (GTK_ACTIONABLE (row), g_variant_new_int32 (num_donation));
 
     external_link_icon = gtk_image_new_from_icon_name ("external-link-symbolic");
-    gtk_widget_add_css_class (external_link_icon, "dim-label");
     adw_action_row_add_suffix (ADW_ACTION_ROW (row), external_link_icon);
 
     adw_expander_row_add_row (self->links_donations, row);
@@ -325,6 +344,16 @@ new_donation_row (ExmDetailView *self,
     self->donation_rows_list = g_list_append (self->donation_rows_list, row);
 }
 
+static void
+delete_donation_rows (ExmDetailView *self)
+{
+  for (GList *iter = self->donation_rows_list; iter != NULL; iter = g_list_next (iter))
+    adw_expander_row_remove (self->links_donations, GTK_WIDGET (iter->data));
+
+  g_list_free (self->donation_rows_list);
+  self->donation_rows_list = NULL;
+}
+
 static void
 update_donation_rows (ExmDetailView  *self,
                       gchar         **donation_urls)
@@ -365,15 +394,17 @@ on_data_loaded (GObject      *source,
     data = exm_data_provider_get_finish (EXM_DATA_PROVIDER (source), result, &error);
     self = EXM_DETAIL_VIEW (user_data);
 
-    if (error && error->code != 404)
+    if (error)
     {
-        // Filter 5xx status codes (server errors)
-        if (error->code / 100 == 5)
-            adw_status_page_set_description (self->error_status, _("Check <a href='https://status.gnome.org/'>GNOME infrastructure status</a> and try again later"));
+        if (error->domain == g_quark_try_string ("request-error-quark") && error->code == 404)
+        {
+            gtk_stack_set_visible_child_name (self->stack, "page_empty");
+        }
         else
-            adw_status_page_set_description (self->error_status, _("Check your network status and try again"));
-
-        gtk_stack_set_visible_child_name (self->stack, "page_error");
+        {
+            adw_status_page_set_description (self->error_status, error->message);
+            gtk_stack_set_visible_child_name (self->stack, "page_error");
+        }
 
         g_clear_error (&error);
 
@@ -415,7 +446,7 @@ on_data_loaded (GObject      *source,
         gtk_label_set_label (self->ext_title, name);
         gtk_label_set_label (self->ext_author, creator);
         gtk_label_set_label (self->ext_description, description);
-        exm_info_bar_set_downloads (self->ext_info_bar, downloads);
+        g_object_set (self->ext_info_bar, "downloads", downloads, NULL);
 
         if (self->resolver_cancel)
         {
@@ -474,7 +505,7 @@ on_data_loaded (GObject      *source,
         adw_action_row_set_subtitle (self->link_homepage, self->uri_homepage);
         adw_action_row_set_subtitle (self->link_extensions, self->uri_extensions);
 
-        exm_info_bar_set_version (self->ext_info_bar, -1);
+        g_object_set (self->ext_info_bar, "version", 0.0, NULL);
 
         for (version_iter = version_map->map;
              version_iter != NULL;
@@ -493,40 +524,37 @@ on_data_loaded (GObject      *source,
               if (version != NULL && self->shell_version != NULL &&
                   (strcmp (version, self->shell_version) == 0 ||
                    strncmp(version, self->shell_version, strchr(version, '.') - version) == 0))
-                  exm_info_bar_set_version (self->ext_info_bar, entry->extension_version);
+                  g_object_set (self->ext_info_bar, "version", entry->extension_version, NULL);
 
             g_free (version);
         }
 
         self->pk = pk;
 
-        if (self->signal_id > 0)
-            g_signal_handler_disconnect (self->show_more_btn, self->signal_id);
+        if (self->comments_signal_id > 0)
+            g_signal_handler_disconnect (self->show_more_btn, self->comments_signal_id);
 
-        self->signal_id = g_signal_connect (self->show_more_btn,
-                                            "clicked",
-                                            G_CALLBACK (show_more_comments),
-                                            self);
+        self->comments_signal_id = g_signal_connect (self->show_more_btn,
+                                                     "clicked",
+                                                     G_CALLBACK (show_more_comments),
+                                                     self);
 
         queue_resolve_comments (self, pk, self->resolver_cancel);
 
-        // Reset scroll position
+        // Reset focus and scroll position
+        gtk_widget_grab_focus (GTK_WIDGET (self->ext_icon));
         gtk_adjustment_set_value (gtk_scrolled_window_get_vadjustment (self->scroll_area), 0);
 
         gtk_stack_set_visible_child_name (self->stack, "page_detail");
 
         return;
     }
-
-    gtk_stack_set_visible_child_name (self->stack, "page_empty");
 }
 
 void
 exm_detail_view_load_for_uuid (ExmDetailView *self,
                                gchar         *uuid)
 {
-    // g_assert (gtk_widget_is_constructed)
-
     self->uuid = uuid;
 
     adw_window_title_set_title (self->title, NULL);
@@ -546,9 +574,7 @@ exm_detail_view_update (ExmDetailView *self)
     // Check if the newly installed extension is the
     // one being displayed in this detail view
     if (exm_manager_is_installed_uuid (self->manager, self->uuid))
-    {
         g_object_set (self->ext_install, "state", EXM_INSTALL_BUTTON_STATE_INSTALLED, NULL);
-    }
 }
 
 static void
@@ -562,9 +588,13 @@ open_link (ExmDetailView *self,
     toplevel = GTK_WIDGET (gtk_widget_get_root (GTK_WIDGET (self)));
 
     if (strcmp (action_name, "detail.open-extensions") == 0)
+    {
         uri = gtk_uri_launcher_new (self->uri_extensions);
+    }
     else if (strcmp (action_name, "detail.open-homepage") == 0)
+    {
         uri = gtk_uri_launcher_new (self->uri_homepage);
+    }
     else if (strcmp (action_name, "detail.open-donation") == 0)
     {
         guint val;
@@ -572,7 +602,9 @@ open_link (ExmDetailView *self,
         uri = gtk_uri_launcher_new (self->uri_donations[val]);
     }
     else
+    {
         g_critical ("open_link() invalid action: %s", action_name);
+    }
 
     gtk_uri_launcher_launch (uri, GTK_WINDOW (toplevel), NULL, NULL, NULL);
 }
@@ -586,6 +618,14 @@ on_bind_manager (ExmDetailView *self)
                   "extensions", &ext_model,
                   NULL);
 
+    if (self->install_signal_id > 0)
+        g_signal_handler_disconnect (self->ext_install, self->install_signal_id);
+
+    self->install_signal_id = g_signal_connect (self->manager,
+                                                "install-status",
+                                                G_CALLBACK (on_install_status),
+                                                self);
+
     g_signal_connect_swapped (ext_model,
                               "items-changed",
                               G_CALLBACK (exm_detail_view_update),
@@ -657,6 +697,7 @@ exm_detail_view_class_init (ExmDetailViewClass *klass)
     gtk_widget_class_bind_template_child (widget_class, ExmDetailView, header_bar);
     gtk_widget_class_bind_template_child (widget_class, ExmDetailView, title);
     gtk_widget_class_bind_template_child (widget_class, ExmDetailView, stack);
+    gtk_widget_class_bind_template_child (widget_class, ExmDetailView, error_label);
     gtk_widget_class_bind_template_child (widget_class, ExmDetailView, error_status);
     gtk_widget_class_bind_template_child (widget_class, ExmDetailView, ext_icon);
     gtk_widget_class_bind_template_child (widget_class, ExmDetailView, ext_title);
@@ -664,8 +705,8 @@ exm_detail_view_class_init (ExmDetailViewClass *klass)
     gtk_widget_class_bind_template_child (widget_class, ExmDetailView, ext_description);
     gtk_widget_class_bind_template_child (widget_class, ExmDetailView, ext_install);
     gtk_widget_class_bind_template_child (widget_class, ExmDetailView, ext_screenshot);
-	gtk_widget_class_bind_template_child (widget_class, ExmDetailView, ext_screenshot_container);
-	gtk_widget_class_bind_template_child (widget_class, ExmDetailView, ext_screenshot_popout_button);
+    gtk_widget_class_bind_template_child (widget_class, ExmDetailView, ext_screenshot_container);
+    gtk_widget_class_bind_template_child (widget_class, ExmDetailView, ext_screenshot_popout_button);
     gtk_widget_class_bind_template_child (widget_class, ExmDetailView, ext_info_bar);
     gtk_widget_class_bind_template_child (widget_class, ExmDetailView, link_homepage);
     gtk_widget_class_bind_template_child (widget_class, ExmDetailView, links_donations);
@@ -678,6 +719,7 @@ exm_detail_view_class_init (ExmDetailViewClass *klass)
     gtk_widget_class_bind_template_callback (widget_class, breakpoint_apply_cb);
     gtk_widget_class_bind_template_callback (widget_class, breakpoint_unapply_cb);
     gtk_widget_class_bind_template_callback (widget_class, screenshot_view_cb);
+    gtk_widget_class_bind_template_callback (widget_class, install_remote);
 
     gtk_widget_class_install_action (widget_class, "detail.open-extensions", NULL, (GtkWidgetActionActivateFunc) open_link);
     gtk_widget_class_install_action (widget_class, "detail.open-homepage", NULL, (GtkWidgetActionActivateFunc) open_link);
@@ -699,11 +741,6 @@ exm_detail_view_init (ExmDetailView *self)
     self->resolver = exm_image_resolver_new ();
     self->comment_provider = exm_comment_provider_new ();
 
-    g_signal_connect (self->ext_install,
-                      "clicked",
-                      G_CALLBACK (install_remote),
-                      self);
-
     adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self->scroll_area));
 
     g_signal_connect_swapped (adj, "value-changed", G_CALLBACK (update_headerbar_cb), self);
