@@ -21,10 +21,11 @@
 
 #include "exm-extension-row.h"
 
+#include "exm-config.h"
 #include "exm-enums.h"
 #include "exm-types.h"
 
-#include "exm-config.h"
+#include <glib/gi18n.h>
 
 struct _ExmExtensionRow
 {
@@ -42,16 +43,15 @@ struct _ExmExtensionRow
     GtkButton *details_btn;
     GtkSwitch *ext_toggle;
 
-    GtkLabel *description_label;
-    GtkLabel *version_title;
-    GtkLabel *version_label;
-    GtkLabel *error_label;
-    GtkLabel *error_label_tag;
+    AdwActionRow *description_row;
+    AdwActionRow *version_row;
+    AdwActionRow *session_modes_row;
+    GtkLabel *info_label;
+    AdwActionRow *error_row;
 
     GtkImage *update_icon;
     GtkImage *error_icon;
     GtkImage *out_of_date_icon;
-    GtkImage *info_icon;
 };
 
 G_DEFINE_FINAL_TYPE (ExmExtensionRow, exm_extension_row, ADW_TYPE_EXPANDER_ROW)
@@ -134,14 +134,6 @@ exm_extension_row_set_property (GObject      *object,
     }
 }
 
-static void
-set_error_label_visible (ExmExtensionRow *self,
-                         gboolean         visible)
-{
-    gtk_widget_set_visible (GTK_WIDGET (self->error_label), visible);
-    gtk_widget_set_visible (GTK_WIDGET (self->error_label_tag), visible);
-}
-
 static gboolean
 transform_to_state (GBinding     *binding G_GNUC_UNUSED,
                     const GValue *from_value,
@@ -163,6 +155,62 @@ unbind_extension (ExmExtensionRow *self)
     }
 }
 
+static void
+add_session_modes (GPtrArray       *session_modes,
+                   ExmExtensionRow *self)
+{
+    if (!session_modes || session_modes->len == 0)
+        return;
+
+    GPtrArray *subtitles = g_ptr_array_new_with_free_func (g_free);
+    gboolean has_unlock_dialog = FALSE;
+    gboolean has_gdm = FALSE;
+
+    for (guint i = 0; i < session_modes->len; i++)
+    {
+        gchar *mode = g_ptr_array_index (session_modes, i);
+
+        if (g_strcmp0 (mode, "unlock-dialog") == 0)
+        {
+            g_ptr_array_add (subtitles, g_strdup (_("Unlock Dialog")));
+            has_unlock_dialog = TRUE;
+        }
+        else if (g_strcmp0 (mode, "gdm") == 0)
+        {
+            // Translators: GNOME Display Manager
+            g_ptr_array_add (subtitles, g_strdup (_("GDM")));
+            has_gdm = TRUE;
+        }
+    }
+
+    if (has_unlock_dialog && has_gdm)
+    {
+        // Translators: Label when an extension runs on both, login and lock screens
+        gtk_label_set_label (self->info_label, _("This extension will run while the screen is locked and no user is logged in"));
+    }
+    else if (has_unlock_dialog)
+    {
+        // Translators: Label when an extension runs on the lock screen
+        gtk_label_set_label (self->info_label, _("This extension will run while the screen is locked"));
+    }
+    else if (has_gdm)
+    {
+        // Translators: Label when an extension runs on the login screen
+        gtk_label_set_label (self->info_label, _("This extension will run while no user is logged in"));
+    }
+
+    if (subtitles->len > 0)
+    {
+        g_ptr_array_add (subtitles, NULL);
+        gchar *subtitle = g_strjoinv (" / ", (gchar **)subtitles->pdata);
+        adw_action_row_set_subtitle (self->session_modes_row, subtitle);
+        gtk_widget_set_visible (GTK_WIDGET (self->session_modes_row), TRUE);
+        g_free (subtitle);
+    }
+
+    g_ptr_array_free (subtitles, TRUE);
+}
+
 static void
 bind_extension (ExmExtensionRow *self,
                 ExmExtension    *extension)
@@ -183,55 +231,45 @@ bind_extension (ExmExtensionRow *self,
     if (self->extension == NULL)
         return;
 
-    gchar *name, *uuid, *description, *version, *version_name, *error_msg;
-    gboolean enabled, has_prefs, has_update, is_user;
+    gchar *name, *uuid, *description, *version, *error;
+    gboolean enabled, has_prefs, is_user;
     ExmExtensionState state;
+    GPtrArray *session_modes;
     g_object_get (self->extension,
-                  "display-name", &name,
+                  "name", &name,
                   "uuid", &uuid,
                   "description", &description,
                   "state", &state,
                   "enabled", &enabled,
-                  "has-prefs", &has_prefs,
-                  "has-update", &has_update,
-                  "is-user", &is_user,
                   "version", &version,
-                  "version-name", &version_name,
-                  "error-msg", &error_msg,
+                  "error", &error,
+                  "has-prefs", &has_prefs,
+                  "is-user", &is_user,
+                  "session-modes", &session_modes,
                   NULL);
 
     self->uuid = g_strdup (uuid);
 
-    g_object_set (self, "title", g_markup_escape_text(name, -1), "subtitle", uuid, NULL);
-    g_object_set (self->prefs_btn, "visible", has_prefs, NULL);
-    g_object_set (self->remove_btn, "visible", is_user, NULL);
-    g_object_set (self->update_icon, "visible", has_update, NULL);
-    g_object_set (self->version_label, "label", version_name ? g_strdup_printf ("%s (%s)", version_name, version)
-                                                             : version, NULL);
+    g_object_set (self, "title", g_markup_escape_text (name, -1), NULL);
 
     // Trim description label's leading and trailing whitespace
     char *description_trimmed = g_strchomp (g_strstrip (description));
-    g_object_set (self->description_label, "label", description_trimmed, NULL);
+    adw_action_row_set_subtitle (self->description_row, description_trimmed);
     g_free (description_trimmed);
 
-    // Only show if error_msg exists and is not empty
-    g_object_set (self->error_label, "label", error_msg, NULL);
-    gboolean has_error = (error_msg != NULL) && (strlen(error_msg) != 0);
-    set_error_label_visible (self, has_error);
+    // Only show if error exists and is not empty
+    gboolean has_error = (error != NULL) && (strlen (error) != 0);
+    gtk_widget_set_visible (GTK_WIDGET (self->error_row), has_error);
 
     gtk_widget_set_visible (GTK_WIDGET (self->error_icon), state == EXM_EXTENSION_STATE_ERROR);
     gtk_widget_set_visible (GTK_WIDGET (self->out_of_date_icon), state == EXM_EXTENSION_STATE_OUT_OF_DATE);
 
-    gtk_widget_set_visible (GTK_WIDGET (self->info_icon),
-                            (state == EXM_EXTENSION_STATE_INITIALIZED
-                             || state == EXM_EXTENSION_STATE_INACTIVE)
-                             && enabled);
-
-    gtk_widget_set_visible (GTK_WIDGET (self->version_title), version != NULL);
-    gtk_widget_set_visible (GTK_WIDGET (self->version_label), version != NULL);
+    gtk_widget_set_visible (GTK_WIDGET (self->version_row), version != NULL);
 
     gtk_actionable_set_action_target (GTK_ACTIONABLE (self->details_btn), "s", uuid);
 
+    add_session_modes (session_modes, self);
+
     GAction *action;
 
     action = g_action_map_lookup_action (G_ACTION_MAP (self->action_group), "open-prefs");
@@ -285,6 +323,12 @@ on_state_changed (GtkSwitch        *toggle,
     return TRUE;
 }
 
+void
+exm_search_row_focus_toggle (ExmExtensionRow *self)
+{
+    gtk_widget_grab_focus (GTK_WIDGET (self->ext_toggle));
+}
+
 static void
 exm_extension_row_class_init (ExmExtensionRowClass *klass)
 {
@@ -314,11 +358,11 @@ exm_extension_row_class_init (ExmExtensionRowClass *klass)
 
     gtk_widget_class_set_template_from_resource (widget_class, g_strdup_printf ("%s/exm-extension-row.ui", RESOURCE_PATH));
 
-    gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, description_label);
-    gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, version_title);
-    gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, version_label);
-    gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, error_label);
-    gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, error_label_tag);
+    gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, description_row);
+    gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, version_row);
+    gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, session_modes_row);
+    gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, info_label);
+    gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, error_row);
 
     gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, prefs_btn);
     gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, remove_btn);
@@ -327,7 +371,6 @@ exm_extension_row_class_init (ExmExtensionRowClass *klass)
     gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, update_icon);
     gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, error_icon);
     gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, out_of_date_icon);
-    gtk_widget_class_bind_template_child (widget_class, ExmExtensionRow, info_icon);
 
     gtk_widget_class_bind_template_callback (widget_class, on_state_changed);
 }
