@@ -11,8 +11,9 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (ExmExtensionRow, exm_extension_row, EXM, EXTENSION_ROW, AdwExpanderRow)
 
-ExmExtensionRow *
-exm_extension_row_new (ExmExtension *extension,
-                       ExmManager   *manager);
+ExmExtensionRow *exm_extension_row_new       (ExmExtension *extension,
+                                              ExmManager   *manager);
+
+void             exm_search_row_focus_toggle (ExmExtensionRow *self);
 
 G_END_DECLS
