@@ -46,6 +46,7 @@ struct _ExmInstalledPage
     GtkFilterListModel *search_list_model;
 
     gboolean sort_enabled_first;
+    guint signal_id;
 };
 
 G_DEFINE_FINAL_TYPE (ExmInstalledPage, exm_installed_page, GTK_TYPE_WIDGET)
@@ -123,18 +124,6 @@ exm_installed_page_set_property (GObject      *object,
     }
 }
 
-GtkStack *
-exm_installed_page_get_stack (ExmInstalledPage *self)
-{
-    return self->stack;
-}
-
-GtkFilterListModel *
-exm_installed_page_get_search_list_model (ExmInstalledPage *self)
-{
-    return self->search_list_model;
-}
-
 static GtkWidget *
 widget_factory (ExmExtension     *extension,
                 ExmInstalledPage *self)
@@ -175,12 +164,12 @@ compare_enabled (ExmExtension *this,
 }
 
 static void
-on_search_changed (GtkSearchEntry *search_entry,
-                   gpointer        user_data)
+on_search_changed (ExmWindow *window,
+                   gpointer   user_data)
 {
     GtkStringFilter *search_filter = (GtkStringFilter *)user_data;
 
-    gtk_string_filter_set_search (search_filter, gtk_editable_get_text (GTK_EDITABLE (search_entry)));
+    gtk_string_filter_set_search (search_filter, exm_window_get_search_query (window));
 }
 
 static void
@@ -188,13 +177,20 @@ on_visible_stack_changed (GObject    *object G_GNUC_UNUSED,
                           GParamSpec *pspec G_GNUC_UNUSED,
                           gpointer    user_data)
 {
-    ExmInstalledPage *self = (ExmInstalledPage *) user_data;
+    ExmInstalledPage *self = (ExmInstalledPage *)user_data;
     ExmWindow *window;
     gboolean search_mode;
 
     window = EXM_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET (self), EXM_TYPE_WINDOW));
-    search_mode = gtk_search_bar_get_search_mode (exm_window_get_search_bar (window));
+    search_mode = exm_window_get_search_mode (window);
 
+    exm_installed_page_show_page (search_mode, self);
+}
+
+void
+exm_installed_page_show_page (gboolean          search_mode,
+                              ExmInstalledPage *self)
+{
     if (search_mode && g_list_model_get_n_items (G_LIST_MODEL (self->search_list_model)) > 0)
         gtk_stack_set_visible_child_name (self->stack , "page_results");
     else if (search_mode)
@@ -214,13 +210,12 @@ bind_list_box (GListModel       *model,
     GtkBoolFilter *is_user_filter;
     GtkFilterListModel *filtered_model;
     ExmWindow *window;
-    GtkSearchEntry *search_entry;
-    const gchar *search_text = NULL;
+    const char *query;
 
     g_return_if_fail (G_IS_LIST_MODEL (model));
 
     // Sort alphabetically
-    expression = gtk_property_expression_new (EXM_TYPE_EXTENSION, NULL, "display-name");
+    expression = gtk_property_expression_new (EXM_TYPE_EXTENSION, NULL, "name");
     alphabetical_sorter = gtk_string_sorter_new (expression);
 
     if (self->sort_enabled_first)
@@ -267,14 +262,13 @@ bind_list_box (GListModel       *model,
                              self, NULL);
 
     window = EXM_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET (self), EXM_TYPE_WINDOW));
-    search_entry = GTK_SEARCH_ENTRY (gtk_search_bar_get_child (exm_window_get_search_bar (window)));
-    search_text = gtk_editable_get_text (GTK_EDITABLE (search_entry));
+    query = exm_window_get_search_query (window);
 
     // Refilter when sort-enabled-first changes and there is an ongoing search
-    if (search_text)
-      gtk_string_filter_set_search (search_filter, search_text);
+    if (query)
+        gtk_string_filter_set_search (search_filter, query);
 
-    g_signal_connect (search_entry,
+    g_signal_connect (window,
                       "search-changed",
                       G_CALLBACK (on_search_changed),
                       search_filter);
@@ -312,6 +306,72 @@ on_updates_available (ExmManager       *manager G_GNUC_UNUSED,
     g_timeout_add (500, G_SOURCE_FUNC (show_updates_banner), self);
 }
 
+static gboolean
+focus_matching_extension (GtkListBox   *list_box,
+                          ExmExtension *extension)
+{
+    int index = 0;
+    ExmExtensionRow *row;
+
+    while ((row = EXM_EXTENSION_ROW (gtk_list_box_get_row_at_index (list_box, index))))
+    {
+        ExmExtension *row_extension;
+
+        g_object_get (row, "extension", &row_extension, NULL);
+
+        if (is_extension_equal (extension, row_extension))
+        {
+            exm_search_row_focus_toggle (row);
+            g_object_unref (row_extension);
+
+            return TRUE;
+        }
+
+        g_object_unref (row_extension);
+        index++;
+    }
+
+    return FALSE;
+}
+
+static void
+on_extensions_changed (GListModel       *model,
+                       guint             position,
+                       guint             removed,
+                       guint             added,
+                       ExmInstalledPage *self)
+{
+    if (!self->sort_enabled_first || (removed > 0 && added > 0))
+        return;
+
+    ExmExtension *extension = EXM_EXTENSION (g_list_model_get_object (model, position));
+
+    if (!extension)
+        return;
+
+    GtkRoot *toplevel = gtk_widget_get_root (GTK_WIDGET (self));
+    GtkWidget *focused_widget = gtk_window_get_focus (GTK_WINDOW (toplevel));
+
+    if (g_list_store_find_with_equal_func (G_LIST_STORE (model), extension, (GEqualFunc)is_extension_equal, &position))
+        g_list_model_items_changed (model, position, 1, 1);
+
+    if (focused_widget && gtk_widget_has_focus (focused_widget)
+        && gtk_widget_get_child_visible (GTK_WIDGET (self)))
+    {
+        if (g_strcmp0 (gtk_stack_get_visible_child_name (self->stack), "page_results") == 0)
+        {
+            focus_matching_extension (self->search_list_box, extension);
+        }
+        else
+        {
+            if (!focus_matching_extension (self->user_list_box, extension))
+                focus_matching_extension (self->system_list_box, extension);
+        }
+    }
+
+    g_object_unref (extension);
+}
+
 static void
 invalidate_model_bindings (ExmInstalledPage *self)
 {
@@ -325,7 +385,17 @@ invalidate_model_bindings (ExmInstalledPage *self)
                   NULL);
 
     if (ext_model)
+    {
         bind_list_box (ext_model, self);
+
+        if (self->signal_id > 0)
+            g_signal_handler_disconnect (ext_model, self->signal_id);
+
+        self->signal_id = g_signal_connect (ext_model,
+                                            "items-changed",
+                                            G_CALLBACK (on_extensions_changed),
+                                            self);
+    }
 }
 
 static void
@@ -392,43 +462,6 @@ exm_installed_page_class_init (ExmInstalledPageClass *klass)
     gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
 }
 
-static GtkWidget *
-create_user_placeholder ()
-{
-    GtkWidget *row, *button, *icon;
-
-    row = adw_action_row_new ();
-    button = gtk_button_new_with_label (_("Browse"));
-    icon = gtk_image_new_from_icon_name ("globe-symbolic");
-
-    gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
-    gtk_widget_set_halign (button, GTK_ALIGN_CENTER);
-
-    gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "win.show-page");
-    gtk_actionable_set_action_target (GTK_ACTIONABLE (button), "s", "browse");
-
-    adw_action_row_add_prefix (ADW_ACTION_ROW (row), icon);
-    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row),
-                                   _("No User Extensions Installed"));
-    adw_action_row_add_suffix (ADW_ACTION_ROW (row), button);
-
-    return row;
-}
-
-static GtkWidget *
-create_system_placeholder ()
-{
-    GtkWidget *row, *icon;
-
-    row = adw_action_row_new ();
-    icon = gtk_image_new_from_icon_name ("settings-symbolic");
-    adw_action_row_add_prefix (ADW_ACTION_ROW (row), icon);
-    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row),
-                                   _("No System Extensions Installed"));
-
-    return row;
-}
-
 static void
 exm_installed_page_init (ExmInstalledPage *self)
 {
@@ -443,10 +476,4 @@ exm_installed_page_init (ExmInstalledPage *self)
                      G_SETTINGS_BIND_GET);
 
     g_object_unref (settings);
-
-    gtk_list_box_set_placeholder (self->user_list_box,
-                                  create_user_placeholder ());
-
-    gtk_list_box_set_placeholder (self->system_list_box,
-                                  create_system_placeholder ());
 }
