@@ -1,6 +1,7 @@
-/* exm-installed-page.h
+/*
+ * exm-installed-page.h
  *
- * Copyright 2022-2024 Matthew Jakeman <mjakeman26@outlook.co.nz>
+ * Copyright 2022-2025 Matthew Jakeman <mjakeman26@outlook.co.nz>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +22,6 @@
 #pragma once
 
 #include <gtk/gtk.h>
-#include <adwaita.h>
 
 G_BEGIN_DECLS
 
@@ -29,13 +29,9 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (ExmInstalledPage, exm_installed_page, EXM, INSTALLED_PAGE, GtkWidget)
 
-ExmInstalledPage *
-exm_installed_page_new (void);
+ExmInstalledPage *exm_installed_page_new       (void);
 
-GtkStack *
-exm_installed_page_get_stack (ExmInstalledPage *self);
-
-GtkFilterListModel *
-exm_installed_page_get_search_list_model (ExmInstalledPage *self);
+void              exm_installed_page_show_page (gboolean          search_mode,
+                                                ExmInstalledPage *self);
 
 G_END_DECLS
