@@ -1,7 +1,30 @@
+/*
+ * exm-screenshot.c
+ *
+ * Copyright 2022-2025 Matthew Jakeman <mjakeman26@outlook.co.nz>
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
+ */
+
 #include "exm-screenshot.h"
 
 #include "exm-zoom-picture.h"
 
+#include "exm-config.h"
+
 struct _ExmScreenshot
 {
     GtkWidget parent_instance;
@@ -12,13 +35,6 @@ struct _ExmScreenshot
 
 G_DEFINE_FINAL_TYPE (ExmScreenshot, exm_screenshot, GTK_TYPE_WIDGET)
 
-enum {
-    PROP_0,
-    N_PROPS
-};
-
-static GParamSpec *properties [N_PROPS];
-
 ExmScreenshot *
 exm_screenshot_new (void)
 {
@@ -37,36 +53,6 @@ exm_screenshot_finalize (GObject *object)
     G_OBJECT_CLASS (exm_screenshot_parent_class)->finalize (object);
 }
 
-static void
-exm_screenshot_get_property (GObject    *object,
-                             guint       prop_id,
-                             GValue     *value,
-                             GParamSpec *pspec)
-{
-    ExmScreenshot *self = EXM_SCREENSHOT (object);
-
-    switch (prop_id)
-    {
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    }
-}
-
-static void
-exm_screenshot_set_property (GObject      *object,
-                             guint         prop_id,
-                             const GValue *value,
-                             GParamSpec   *pspec)
-{
-    ExmScreenshot *self = EXM_SCREENSHOT (object);
-
-    switch (prop_id)
-    {
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    }
-}
-
 void
 exm_screenshot_set_paintable (ExmScreenshot *self,
                               GdkPaintable  *paintable)
@@ -114,7 +100,7 @@ exm_screenshot_measure (GtkWidget      *widget,
 }
 
 static GtkSizeRequestMode
-exm_screenshot_get_request_mode (GtkWidget *widget)
+exm_screenshot_get_request_mode (GtkWidget *widget G_GNUC_UNUSED)
 {
     return GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
 }
@@ -138,12 +124,10 @@ exm_screenshot_class_init (ExmScreenshotClass *klass)
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
     object_class->finalize = exm_screenshot_finalize;
-    object_class->get_property = exm_screenshot_get_property;
-    object_class->set_property = exm_screenshot_set_property;
 
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-    gtk_widget_class_set_template_from_resource (widget_class, "/com/mattjakeman/ExtensionManager/exm-screenshot.ui");
+    gtk_widget_class_set_template_from_resource (widget_class, g_strdup_printf ("%s/exm-screenshot.ui", RESOURCE_PATH));
 
     widget_class->get_request_mode = exm_screenshot_get_request_mode;
     widget_class->measure = exm_screenshot_measure;
@@ -154,6 +138,8 @@ exm_screenshot_class_init (ExmScreenshotClass *klass)
     gtk_widget_class_bind_template_child (widget_class, ExmScreenshot, picture);
     gtk_widget_class_bind_template_child (widget_class, ExmScreenshot, stack);
 
+    gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_IMG);
+
 }
 
 static void
