@@ -1,3 +1,24 @@
+/*
+ * exm-search-provider.c
+ *
+ * Copyright 2022-2025 Matthew Jakeman <mjakeman26@outlook.co.nz>
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
+ */
+
 #include "exm-search-provider.h"
 
 #include "model/exm-search-result.h"
@@ -34,14 +55,6 @@ exm_search_provider_new (void)
     return g_object_new (EXM_TYPE_SEARCH_PROVIDER, NULL);
 }
 
-static void
-exm_search_provider_finalize (GObject *object)
-{
-    ExmSearchProvider *self = (ExmSearchProvider *)object;
-
-    G_OBJECT_CLASS (exm_search_provider_parent_class)->finalize (object);
-}
-
 static void
 exm_search_provider_get_property (GObject    *object,
                                   guint       prop_id,
@@ -235,7 +248,6 @@ exm_search_provider_class_init (ExmSearchProviderClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-    object_class->finalize = exm_search_provider_finalize;
     object_class->get_property = exm_search_provider_get_property;
     object_class->set_property = exm_search_provider_set_property;
 
