@@ -1,5 +1,27 @@
+/* exm-comment.c
+ *
+ * Copyright 2022-2024 Matthew Jakeman <mjakeman26@outlook.co.nz>
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
+ */
+
 #include "exm-comment.h"
 
+#include "exm-utils.h"
+
 #include <json-glib/json-glib.h>
 
 struct _ExmComment
@@ -10,6 +32,7 @@ struct _ExmComment
     gchar *comment;
     gchar *author;
     int rating;
+    gchar *date;
 };
 
 static JsonSerializableIface *serializable_iface = NULL;
@@ -26,6 +49,7 @@ enum {
     PROP_COMMENT,
     PROP_AUTHOR,
     PROP_RATING,
+    PROP_DATE,
     N_PROPS
 };
 
@@ -37,14 +61,6 @@ exm_comment_new (void)
     return g_object_new (EXM_TYPE_COMMENT, NULL);
 }
 
-static void
-exm_comment_finalize (GObject *object)
-{
-    ExmComment *self = (ExmComment *)object;
-
-    G_OBJECT_CLASS (exm_comment_parent_class)->finalize (object);
-}
-
 static void
 exm_comment_get_property (GObject    *object,
                           guint       prop_id,
@@ -59,7 +75,7 @@ exm_comment_get_property (GObject    *object,
         g_value_set_boolean (value, self->is_extension_creator);
         break;
     case PROP_COMMENT:
-        g_value_set_string (value, self->comment);
+        g_value_set_string (value, exm_utils_convert_html (self->comment));
         break;
     case PROP_AUTHOR:
         g_value_set_string (value, self->author);
@@ -67,6 +83,9 @@ exm_comment_get_property (GObject    *object,
     case PROP_RATING:
         g_value_set_int (value, self->rating);
         break;
+    case PROP_DATE:
+        g_value_set_string (value, self->date);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -94,6 +113,9 @@ exm_comment_set_property (GObject      *object,
     case PROP_RATING:
         self->rating = g_value_get_int (value);
         break;
+    case PROP_DATE:
+        self->date = g_value_dup_string (value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -104,7 +126,6 @@ exm_comment_class_init (ExmCommentClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-    object_class->finalize = exm_comment_finalize;
     object_class->get_property = exm_comment_get_property;
     object_class->set_property = exm_comment_set_property;
 
@@ -136,6 +157,13 @@ exm_comment_class_init (ExmCommentClass *klass)
                           -1, 5, -1, /* min -1, max 5, default -1 */
                           G_PARAM_READWRITE);
 
+    properties [PROP_DATE] =
+        g_param_spec_string ("date",
+                             "Date",
+                             "Date",
+                             NULL,
+                             G_PARAM_READWRITE);
+
     g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
@@ -161,6 +189,14 @@ exm_comment_deserialize_property (JsonSerializable *serializable,
         g_value_set_string (value, json_object_get_string_member (obj, "username"));
         return TRUE;
     }
+    else if (strcmp (property_name, "date") == 0)
+    {
+        JsonObject *obj;
+
+        obj = json_node_get_object (property_node);
+        g_value_set_string (value, json_object_get_string_member (obj, "timestamp"));
+        return TRUE;
+    }
 
     return serializable_iface->deserialize_property (serializable,
                                                      property_name,
